@@ -46,6 +46,7 @@ class DeploymentClient(object):
     def __init__(self, socket_path):
         self.socket_path = socket_path
         self._buffer = []
+        self.exit_status = 0
 
         # Connect to unix socket
         self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
@@ -83,6 +84,9 @@ class DeploymentClient(object):
         try:
             tmux_env = os.environ.get('TMUX', '')
             xterm_env = os.environ.get('XTERM', '')
+            display_env = os.environ.get('DISPLAY', '')
+            colorterm_env = os.environ.get('COLORTERM', '')
+
             if tmux_env:
                 # Construct tmux split command
                 swap = (' && (tmux last-pane || true)' if not focus else '')
@@ -93,15 +97,16 @@ class DeploymentClient(object):
                     # pane, that we use the same virtualenv for this command.
                 path_env = os.environ.get('PATH', '')
 
-                subprocess.call(r'TMUX=%s tmux split-window "PATH=\"%s\" %s" %s %s' % (tmux_env, path_env, self.new_window_command, swap, tiled),
+                subprocess.call(r'TMUX=%s tmux split-window "PATH=\"%s\" %s" %s %s' %
+                        (tmux_env, path_env, self.new_window_command, swap, tiled),
                         shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            elif xterm_env:
-                # When in a gnome-terminal:
-                if os.environ.get('COLORTERM', '') == 'gnome-terminal':
-                    subprocess.call('gnome-terminal -e "%s" &' % self.new_window_command, shell=True)
-                # Fallback to xterm
-                else:
-                    subprocess.call('xterm -e %s &' % self.new_window_command, shell=True)
+             
+            # When in a gnome-terminal:
+            elif display_env and colorterm_env == 'gnome-terminal':
+                subprocess.call('gnome-terminal -e "%s" &' % self.new_window_command, shell=True)
+	    # Fallback to xterm
+            elif display_env and xterm_env:
+                subprocess.call('xterm -e %s &' % self.new_window_command, shell=True)
             else:
                 # Failed, print err.
                 sys.stdout.write(
@@ -142,15 +147,18 @@ class DeploymentClient(object):
             elif action == '_info':
                 print termcolor.colored(self.socket_path, 'cyan')
                 print '     Created:             %s' % data['created']
-                print '     Root service name:   %s' % data['root_service_name']
-                print '     Root service module: %s' % data['root_service_module']
+                print '     Root node name:      %s' % data['root_node_name']
+                print '     Root node module:    %s' % data['root_node_module']
                 print '     Processes: (%i)' % len(data['processes'])
 
                 for i, process in enumerate(data['processes']):
                     print '     %i' % i
-                    print '     - Service name    %s' % process['service_name']
-                    print '     - Service module  %s' % process['service_module']
-                    print '     - Running         %s' % process['running']
+                    print '     - Node name    %s' % process['node_name']
+                    print '     - Node module  %s' % process['node_module']
+                    print '     - Running      %s' % process['running']
+
+            elif action == 'set-exit-status':
+                self.exit_status = data
 
             # Keep the remainder for the next time
             remainder = io.read()
@@ -167,7 +175,7 @@ class DeploymentClient(object):
         self.socket.sendall(pickle.dumps(('_get_info', '')))
         self._read_loop()
 
-    def run(self, cd_path=None):
+    def run(self, cd_path=None, action_name=None, parameters=None):
         """
         Run main event loop.
         """
@@ -181,14 +189,21 @@ class DeploymentClient(object):
 
         self.socket.sendall(pickle.dumps(('_start-interaction', {
                 'cd_path': cd_path,
+                'action_name': action_name,
+                'parameters': parameters,
             })))
 
         self._read_loop()
-        sys.stdout.write('\n')
 
         # Reset terminal state
         termios.tcsetattr(sys.stdin, termios.TCSAFLUSH, tcattr)
 
+        # Put the cursor again at the left margin.
+        sys.stdout.write('\r\n')
+
+        # Set exit status
+        sys.exit(self.exit_status)
+
     def _read_loop(self):
         while True:
             try:
@@ -233,9 +248,10 @@ def list_sessions():
             pass
 
 
-def start(socket_name, cd_path=None):
+def start(socket_name, cd_path=None, action_name=None, parameters=None):
     """
     Start a socket client.
     """
     make_stdin_unbuffered()
-    DeploymentClient(socket_name).run(cd_path=cd_path)
+    DeploymentClient(socket_name).run(cd_path=cd_path,
+                    action_name=action_name, parameters=parameters)
