@@ -2,7 +2,6 @@ import sys
 import threading
 import termios
 import tty
-import pty
 
 class TeeStd(object):
     """
@@ -112,13 +111,17 @@ class raw_mode(object):
     """
     def __init__(self, stdin):
         self.stdin = stdin
-        self.attrs_before = termios.tcgetattr(self.stdin)
+
+        if self.stdin.isatty():
+            self.attrs_before = termios.tcgetattr(self.stdin)
 
     def __enter__(self):
-        # NOTE: On os X systems, using pty.setraw() fails. Therefor we are using this:
-        newattr = termios.tcgetattr(self.stdin.fileno())
-        newattr[tty.LFLAG] = newattr[tty.LFLAG] & ~(termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
-        termios.tcsetattr(self.stdin.fileno(), termios.TCSANOW, newattr)
+        if self.stdin.isatty():
+            # NOTE: On os X systems, using pty.setraw() fails. Therefor we are using this:
+            newattr = termios.tcgetattr(self.stdin.fileno())
+            newattr[tty.LFLAG] = newattr[tty.LFLAG] & ~(termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
+            termios.tcsetattr(self.stdin.fileno(), termios.TCSANOW, newattr)
 
     def __exit__(self, *a, **kw):
-        termios.tcsetattr(self.stdin.fileno(), termios.TCSANOW, self.attrs_before)
+        if self.stdin.isatty():
+            termios.tcsetattr(self.stdin.fileno(), termios.TCSANOW, self.attrs_before)
